package `in`.finbox.deviceconnect.finbox_dc_plugin

import `in`.finbox.mobileriskmanager.FinBox
import `in`.finbox.mobileriskmanager.common.annotations.FinBoxErrorCode
import `in`.finbox.mobileriskmanager.devicematch.DeviceMatch
import `in`.finbox.mobileriskmanager.notifications.MessagingService
import android.content.Context
import android.os.Handler
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import android.os.Looper

/** FinBoxDcPlugin */
class FinBoxDcPlugin : FlutterPlugin, MethodCallHandler {
    companion object {
        fun initLibrary(context: Context) {
            FinBox.initLibrary(context)
        }
    }

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var deviceConnectChannel: MethodChannel
    private lateinit var finbox: FinBox
    private lateinit var context: Context
    private val handler: Handler = Handler(Looper.getMainLooper())

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext
        deviceConnectChannel =
            MethodChannel(flutterPluginBinding.binaryMessenger, "in.finbox.lending/deviceconnect")
        finbox = FinBox()
        deviceConnectChannel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "createUser" -> {
                FinBox.createUser(
                    call.argument("api_key")!!, call.argument("customer_id")!!,
                    object : FinBox.FinBoxAuthCallback {
                        override fun onSuccess(accessToken: String) {
                            // Set up the data Sync
                            result.success(accessToken)
                        }

                        override fun onError(@FinBoxErrorCode errorCode: Int) {
                            result.error(
                                errorCode.toString(),
                                "Refer https://docs.finbox.in/device-connect/error-codes.html",
                                errorCode
                            )
                        }
                    })
            }
            "startPeriodicSync" -> {
                finbox.startPeriodicSync()
            }
            "stopPeriodicSync" -> {
                finbox.stopPeriodicSync()
            }
            "setSyncFrequency" -> {
                call.argument<Int>("frequency")?.let {
                    finbox.setSyncFrequency(it.toLong())
                }
            }
            "resetData" -> {
                FinBox.resetData()
            }
            "setDeviceMatch" -> {
                val deviceMatch = DeviceMatch.Builder().apply {
                    setEmail(call.argument<String>("email")!!)
                    setName(call.argument<String>("userName")!!)
                    setPhone(call.argument<String>("phone")!!)
                }.build()
                finbox.setDeviceMatch(deviceMatch)
            }
            "forwardToFinBoxSDK" -> {
                // Get the message data
                val data = call.arguments as Map<String, String>
                if (MessagingService.forwardToFinBoxSDK(data)) {
                    // FinBox Firebase Message
                    val firebaseMessagingService = MessagingService(context)
                    Thread {
                        firebaseMessagingService.onMessageReceived(data)
                    }.run()
                } else {
                    // Nothing to do
                    // Irrelevant to FinBox SDk
                }
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        deviceConnectChannel.setMethodCallHandler(null)
    }
}
